import { jsx as _jsx } from "react/jsx-runtime";
import Input, {} from "@mui/material/Input";
import { forwardRef, useCallback, useContext, useEffect, useState } from "react";
import { NumberFormatContext } from "../NumberFormatContext/index.js";
import { usePrevious } from "../utils/react.js";
export const DEFAULT_DECIMAL_PLACES = 16;
/**
 * A number input component that leverages the NumberFormatContext for
 * formatting and parsing. Default formatting and parsing can be overridden by
 * wrapping this component in a NumberFormatContext.Provider.
 */
const NumberInput = forwardRef(({ onChange, onBlur, value, defaultValue, max, min, error = false, onError, onErrorEnd, allowUndefined, correctOnBlur = true, maxDecimalPlaces = DEFAULT_DECIMAL_PLACES, ...props }, ref) => {
    const { formatNumber, parseNumber } = useContext(NumberFormatContext);
    const [textValue, setTextValue] = useState(value === undefined ? "" : formatNumber(value));
    const [numericValue, setNumericValue] = useState(value ?? NaN);
    const [lastValid, setLastValid] = useState(numericValue);
    const isValidNaN = useCallback((val) => allowUndefined && !val?.trim(), [allowUndefined]);
    const isOutOfRange = useCallback((val) => (typeof min === "number" && val < min) || (typeof max === "number" && val > max), [min, max]);
    const isTooPrecise = useCallback((val) => getDecimalPlaces(val) > maxDecimalPlaces, [maxDecimalPlaces]);
    const invalidError = isValidNaN(textValue)
        ? false
        : // If the user begins to type a negative number, do not present an invalid
            // error.
            textValue?.trim() !== "-" && parseNumber(textValue) !== numericValue;
    const previousError = usePrevious(error || invalidError);
    useEffect(() => {
        if (!isNaN(numericValue) &&
            !isOutOfRange(numericValue) &&
            !isTooPrecise(numericValue)) {
            setLastValid(numericValue);
        }
    }, [numericValue, isOutOfRange, isTooPrecise]);
    useEffect(() => {
        const e = error || invalidError;
        if (e !== previousError) {
            if (invalidError) {
                const inputVal = parseNumber(textValue);
                let type;
                if (isNaN(numericValue)) {
                    type = "NaN";
                }
                else if (isOutOfRange(inputVal)) {
                    type = "out-of-range";
                }
                else if (isTooPrecise(inputVal)) {
                    type = "max-decimals";
                }
                else {
                    type = "unknown";
                }
                onError?.(numericValue, textValue, type);
            }
            else if (error) {
                onError?.(numericValue, textValue, "unknown");
            }
            else {
                onErrorEnd?.(numericValue, textValue);
            }
        }
    }, [
        error,
        previousError,
        numericValue,
        textValue,
        onError,
        onErrorEnd,
        invalidError,
        isOutOfRange,
        isTooPrecise,
        parseNumber,
    ]);
    const reformat = useCallback(() => {
        if ((isNaN(numericValue) && !isValidNaN(textValue)) || isOutOfRange(numericValue)) {
            setNumericValue(lastValid);
            setTextValue(formatNumber(lastValid));
            onChange?.(lastValid);
        }
        else if (!isNaN(numericValue)) {
            setTextValue(formatNumber(numericValue));
        }
    }, [formatNumber, isOutOfRange, isValidNaN, lastValid, numericValue, onChange, textValue]);
    const correctValue = useCallback((val) => {
        let newValue = val;
        if (!isNaN(newValue) && typeof newValue === "number") {
            if (typeof max === "number") {
                newValue = Math.min(newValue, max);
            }
            if (typeof min === "number") {
                newValue = Math.max(newValue, min);
            }
            const decimalPlaces = getDecimalPlaces(newValue);
            const maxDecimals = Math.min(Math.max(0, maxDecimalPlaces), DEFAULT_DECIMAL_PLACES);
            if (decimalPlaces > maxDecimals) {
                newValue = +newValue.toFixed(maxDecimals);
            }
        }
        return newValue;
    }, [max, min, maxDecimalPlaces]);
    useEffect(() => {
        if (value === undefined) {
            setTextValue("");
            setNumericValue(NaN);
        }
        else {
            // For controlled components, only update the text from the value if
            // it's a valid number, and represents a number different from the
            // one in the text box. Otherwise leave it alone so that it's not
            // constantly reformatting as the user is typing.
            if (!isNaN(value) && value !== numericValue) {
                setTextValue(formatNumber(value));
            }
            setNumericValue(correctValue(value));
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps -- It's intentional to omit numericValue. We only want this to run when the incoming value changes, not when numericValue changes in handleChange().
    }, [value, formatNumber, correctValue]);
    const handleChange = useCallback(event => {
        const value = event.currentTarget.value;
        setTextValue(value);
        if (isValidNaN(value)) {
            setNumericValue(NaN);
            onChange?.(undefined);
        }
        else {
            const newValue = correctValue(parseNumber(value));
            setNumericValue(newValue);
            onChange?.(newValue);
        }
    }, [correctValue, onChange, parseNumber, isValidNaN]);
    const handleBlur = useCallback((event) => {
        if (correctOnBlur) {
            reformat();
        }
        onBlur?.(event);
    }, [correctOnBlur, reformat, onBlur]);
    return (_jsx(Input, { ref: ref, type: "text", value: textValue, onChange: handleChange, onBlur: handleBlur, defaultValue: typeof defaultValue === "number" ? formatNumber(defaultValue) : undefined, "data-test": "NumberInput-container", error: error || invalidError, ...props }));
});
export default NumberInput;
function getDecimalPlaces(value) {
    const text = value.toString();
    if (text.includes("e-")) {
        const [, exponent] = text.split("e-");
        return parseInt(exponent, 10);
    }
    if (Math.floor(value) !== value) {
        const [, decimals] = text.split(".");
        return decimals.length ?? 0;
    }
    return 0;
}
