import { jsx as _jsx } from "react/jsx-runtime";
import SvgIcon from "@mui/material/SvgIcon";
import baseCreateSvgIcon from "@mui/material/utils/createSvgIcon";
import clsx from "clsx";
import { forwardRef, memo } from "react";
import { styled, mergeStyles, useTheme } from "../../styles/index.js";
const PREFIX = "GcxSvgIcon";
const gcxSvgIconClasses = {
    mirrored: `${PREFIX}-mirrored`,
};
/**
 * Creates an SVG icon component from a <path>.
 *
 * @param path The React component representing the inner path of the SVG.
 * @param displayName The display name.
 * @param mirror Whether or not to mirror the icon in RTL mode. By default it
 *   will mirror anything with "Left" or "Right" in the display name, as well as
 *   some built-in icons. See the Material UI guidelines for a discussion of
 *   which icons should be mirrored.
 */
export default function createSvgIcon(path, displayName, mirror) {
    const Icon = baseCreateSvgIcon(path, displayName);
    const StyledIcon = styled(Icon)({
        [`&.${gcxSvgIconClasses.mirrored}`]: {
            transform: "scaleX(-1)",
        },
    });
    // Create a wrapper that can mirror the icon horizontally if the direction
    // is RTL.
    const RtlWrapper = (props, ref) => {
        const { direction } = useTheme();
        const { classes, className, ...otherProps } = props;
        const { mirrored: mirroredClass, ...otherClasses } = mergeStyles(gcxSvgIconClasses, classes);
        return (_jsx(StyledIcon, { classes: otherClasses, className: clsx(className, {
                [mirroredClass]: direction === "rtl" &&
                    mirror !== false &&
                    (mirror ?? shouldMirror(displayName)),
            }), ref: ref, ...otherProps }));
    };
    // Copied from MUI implementation.
    if (process.env.NODE_ENV !== "production") {
        // Need to set `displayName` on the inner component for React.memo.
        // React prior to 16.14 ignores `displayName` on the wrapper.
        RtlWrapper.displayName = `${displayName}Icon`;
    }
    RtlWrapper.muiName = SvgIcon.muiName;
    return memo(forwardRef(RtlWrapper));
}
function shouldMirror(displayName) {
    // See here for a discussion of which icons should be mirrored:
    // https://material.io/design/usability/bidirectionality.html#mirroring-elements
    return /^(.*Left.*|.*Right.*|Accessibility|ArcgisShortcut|AlignToEdge|Arrow|ArrowBack|Callout|Cards|Chart.*|Details|DetailsDropdown|DrawArrow|Export|ExtentScale|FeatureSummary|IdGenerate|LayersExtract|Legend|LinkExternal|MapBing|MapGoogle|Maximize|MaximizeRestore|Menu|MenuGlobal|MenuHoisted|Message|Metadata|Minimize|MinimizeRestore|Pictometry|Playback.*|PoolsSearch|PopIn|PopOut|Portal|Profile|Redo|RangeEnd|RangeStart|ResultCard|ResultClear.*|ResultList|ResultPan|ReturnArrow|Selected(All|None|Partial)|ShareMap|Text|Undo|ViewList|WindowDock|WindowNew|XlsxExport|ZoomFeature|ZoomNext|ZoomPrevious)$/.test(displayName);
}
