import { alpha, createTheme as createMuiTheme } from "@mui/material/styles";
import { defaultPalette, getPaletteOverrides } from "./palette.js";
import blue from "../colors/blue.js";
const fontWeightLightest = 200;
const fontWeightLight = 300;
const fontWeightRegular = 400;
const fontWeightMedium = 600;
const fontWeightBold = 700;
const getDefaultOptions = (theme) => {
    const { palette, typography: { pxToRem }, } = theme;
    return {
        palette: { ...defaultPalette, ...getPaletteOverrides(palette) },
        // Defaults should match https://meridian.vertigis.com/docs/design-language/#typography
        // Use `pxToRem` so clients with different root font sizes still work properly.
        typography: {
            fontFamily: '"Segoe UI", "Helvetica Neue", "Roboto", Helvetica, Arial, sans-serif',
            fontWeightLight,
            fontWeightRegular,
            fontWeightMedium,
            fontWeightBold,
            h1: {
                fontSize: pxToRem(32),
                fontWeight: fontWeightLight,
                lineHeight: 1.25,
            },
            h2: {
                fontSize: pxToRem(26),
                fontWeight: fontWeightLight,
                lineHeight: 1.25,
            },
            h3: {
                fontSize: pxToRem(22),
                fontWeight: fontWeightLightest,
                lineHeight: 1.25,
            },
            h4: {
                fontSize: pxToRem(20),
                fontWeight: fontWeightRegular,
                lineHeight: 1.25,
            },
            h5: {
                fontSize: pxToRem(18),
                fontWeight: fontWeightMedium,
                lineHeight: 1.25,
            },
            h6: {
                fontSize: pxToRem(16),
                fontWeight: fontWeightMedium,
                lineHeight: 1.125,
            },
            subtitle1: {
                fontSize: pxToRem(16),
                fontWeight: fontWeightRegular,
                lineHeight: 1.125,
                letterSpacing: pxToRem(0.15),
            },
            subtitle2: {
                fontSize: pxToRem(14),
                fontWeight: fontWeightMedium,
                lineHeight: 1.5,
                letterSpacing: pxToRem(0.17),
            },
            body1: {
                fontSize: pxToRem(14),
                fontWeight: fontWeightRegular,
                lineHeight: 1.5,
            },
            body2: {
                fontSize: pxToRem(12),
                fontWeight: fontWeightRegular,
                lineHeight: 1.375,
            },
            caption: {
                fontSize: pxToRem(14),
                fontWeight: fontWeightRegular,
                fontStyle: "italic",
                lineHeight: 1.625,
            },
            button: {
                fontSize: pxToRem(14),
                fontWeight: fontWeightRegular,
                lineHeight: 1.75,
                textTransform: "none",
            },
        },
    };
};
const defaultDenseOptions = {
    spacing: 5,
    components: {
        MuiButton: {
            defaultProps: { size: "small" },
        },
        MuiCheckbox: {
            defaultProps: { size: "small" },
        },
        MuiFilledInput: {
            defaultProps: { margin: "dense" },
        },
        MuiFormControl: {
            defaultProps: { margin: "dense" },
        },
        MuiFormHelperText: {
            defaultProps: { margin: "dense" },
        },
        MuiIconButton: {
            defaultProps: { size: "small" },
        },
        MuiInputBase: {
            defaultProps: { margin: "dense" },
        },
        MuiInputLabel: {
            defaultProps: { margin: "dense" },
        },
        MuiListItem: {
            defaultProps: { dense: true },
        },
        MuiListItemButton: {
            defaultProps: { dense: true },
        },
        MuiMenuItem: {
            defaultProps: { dense: true },
        },
        MuiOutlinedInput: {
            defaultProps: { margin: "dense" },
        },
        MuiFab: {
            defaultProps: { size: "small" },
        },
        MuiSwitch: {
            defaultProps: { size: "small" },
        },
        MuiSvgIcon: {
            defaultProps: { fontSize: "small" },
        },
        MuiTable: {
            defaultProps: { size: "small" },
        },
        MuiTextField: {
            defaultProps: { margin: "dense" },
        },
        MuiToolbar: {
            defaultProps: { variant: "dense" },
        },
    },
};
/**
 * Common style overrides for both Select and NativeSelect.
 */
const getSelectStyles = (theme) => ({
    borderRadius: theme.shape.borderRadius,
    "&:focus": {
        borderRadius: theme.shape.borderRadius,
        // Reset focused background color
        backgroundColor: "inherit",
    },
});
const getFocusHighlightStyles = ({ palette }) => ({
    boxShadow: `0 0 0 2px ${palette.grey[600]} inset,
        0 0 0 3px ${palette.common.white} inset,
        0 0 0 5px ${palette.grey[600]} inset`,
});
/**
 * Common style overrides for both MenuItem and ListItemButton.
 */
const getMenuItemStyles = ({ palette, transitions }) => ({
    overflow: "hidden",
    "&:hover": {
        backgroundColor: "transparent",
    },
    "&:before": {
        content: '""',
        position: "absolute",
        left: 0,
        display: "block",
        width: 3,
        height: "100%",
        backgroundColor: palette.primary.main,
        transform: "translateX(-3px)",
        transition: transitions.create(["transform"], {
            duration: transitions.duration.short,
        }),
    },
    "&:hover:before": {
        transform: "translateX(0)",
    },
    "&.Mui-selected:before": {
        width: 6,
        backgroundColor: palette.primary.main,
        transform: "scaleX(1.5)",
    },
    "&.Mui-selected:hover": {
        backgroundColor: "transparent",
    },
    "&:hover, &.Mui-selected": {
        background: `linear-gradient(to right, ${palette.custom.greyBackground} 0%, transparent 100%)`,
    },
});
/**
 * The overrides may be based on the theme's palette. We must create these
 * separately after the fact.
 */
function getComponentOverrides(theme, isDense) {
    const { palette, shape, spacing, transitions, typography: { pxToRem, fontWeightMedium, fontWeightBold, body1, subtitle2 }, } = theme;
    return {
        components: {
            MuiAutocomplete: {
                styleOverrides: {
                    root: {
                        ".MuiInput-root": {
                            ".MuiInput-input": {
                                // Let the height expand to fit content
                                height: "auto",
                                padding: `${spacing(0.625)} ${spacing(1)}`,
                            },
                        },
                    },
                },
            },
            MuiBackdrop: {
                styleOverrides: {
                    root: {
                        backgroundColor: "rgba(0, 0, 0, 0.4)",
                    },
                    invisible: {
                        backgroundColor: "transparent",
                    },
                },
            },
            MuiButton: {
                defaultProps: {
                    disableFocusRipple: true,
                    color: "grey",
                    variant: "contained",
                },
                styleOverrides: {
                    // Button will use 'actionable' palette instead of primary
                    // for primary/contrast button colors This allows the app to
                    // have different colors for primary components (used in
                    // sidebar, icons, etc) and actionable components (buttons)
                    root: {
                        borderStyle: "solid",
                        borderWidth: 1,
                        "&.Mui-focusVisible:active": {
                            backgroundColor: alpha(palette.grey.A200, 0.5),
                        },
                        variants: [
                            {
                                // The "grey" color is our own augmentation which is
                                // equivalent to the "default" color in MUI v4, which
                                // was removed in v5.
                                props: { color: "grey", variant: "contained" },
                                style: {
                                    color: palette.getContrastText(palette.grey[300]),
                                },
                            },
                            {
                                props: { color: "grey", variant: "outlined" },
                                style: {
                                    color: palette.text.primary,
                                    borderColor: palette.custom.greyOutline,
                                    "&.Mui-disabled": {
                                        border: `1px solid ${palette.action.disabledBackground}`,
                                    },
                                    "&:hover": {
                                        borderColor: palette.custom.greyOutline,
                                        backgroundColor: alpha(palette.text.primary, palette.action.hoverOpacity),
                                    },
                                },
                            },
                            {
                                props: { color: "grey", variant: "text" },
                                style: {
                                    color: palette.text.primary,
                                    "&:hover": {
                                        backgroundColor: alpha(palette.text.primary, palette.action.hoverOpacity),
                                    },
                                },
                            },
                        ],
                    },
                    contained: {
                        borderColor: palette.grey[400],
                        boxShadow: "none",
                        "&:hover": {
                            boxShadow: "none",
                        },
                        "&.Mui-disabled": {
                            borderColor: palette.grey[300],
                        },
                        "&:active": {
                            boxShadow: "none",
                        },
                        "&.Mui-focusVisible": {
                            ...getFocusHighlightStyles(theme),
                        },
                        "&.Mui-focusVisible:active": {
                            backgroundColor: palette.grey[500],
                        },
                    },
                    containedPrimary: {
                        borderColor: palette.primary.dark,
                        "&.Mui-focusVisible:active": {
                            backgroundColor: alpha(palette.primary.main, 0.5),
                        },
                    },
                    containedSecondary: {
                        borderColor: palette.secondary.dark,
                        "&.Mui-focusVisible:active": {
                            backgroundColor: alpha(palette.secondary.main, 0.5),
                        },
                    },
                    outlined: {
                        "&.Mui-focusVisible:active": {
                            backgroundColor: palette.grey[500],
                        },
                    },
                    sizeMedium: {
                        padding: `${spacing(0.75)} ${spacing(2)}`,
                    },
                    text: {
                        borderColor: "transparent",
                    },
                    textPrimary: {
                        "&.Mui-focusVisible:active": {
                            backgroundColor: alpha(palette.primary.main, 0.25),
                        },
                    },
                    textSecondary: {
                        "&.Mui-focusVisible:active": {
                            backgroundColor: alpha(palette.secondary.main, 0.25),
                        },
                    },
                },
            },
            MuiButtonBase: {
                styleOverrides: {
                    root: {
                        "&.Mui-focusVisible": {
                            ...getFocusHighlightStyles(theme),
                        },
                    },
                },
            },
            MuiButtonGroup: {
                defaultProps: {
                    color: "grey",
                    disableFocusRipple: true,
                    variant: "contained",
                },
                styleOverrides: {
                    contained: {
                        boxShadow: "none",
                    },
                    grouped: {
                        margin: 0,
                    },
                    groupedContained: {
                        "&:not(:first-child)": {
                            // Hide double border
                            borderLeftWidth: 0,
                        },
                    },
                },
            },
            MuiCheckbox: {
                defaultProps: {
                    color: "secondary",
                    focusRipple: false,
                },
            },
            MuiCssBaseline: {
                styleOverrides: {
                    html: {
                        WebkitFontSmoothing: "antialiased", // Antialiasing.
                        MozOsxFontSmoothing: "grayscale", // Antialiasing.
                        // Change from `box-sizing: content-box` so that `width`
                        // is not affected by `padding` or `border`.
                        boxSizing: "border-box",
                    },
                    "*, *::before, *::after": {
                        boxSizing: "inherit",
                    },
                    "strong, b": {
                        fontWeight: fontWeightBold,
                    },
                    body: {
                        margin: 0,
                        padding: 0,
                        color: palette.text.primary,
                        ...body1,
                        backgroundColor: palette.background.default,
                        overflow: "hidden",
                        "@media print": {
                            // Save printer ink.
                            backgroundColor: palette.common.white,
                        },
                        // Add support for document.body.requestFullScreen().
                        // Other elements, if background transparent, are not supported.
                        "&::backdrop": {
                            backgroundColor: palette.background.default,
                        },
                    },
                },
            },
            MuiDatePicker: {
                defaultProps: {
                    slotProps: {
                        openPickerButton: { size: "small" },
                        popper: { dir: theme.direction },
                    },
                },
            },
            MuiDateRangePicker: {
                defaultProps: {
                    slotProps: { popper: { dir: theme.direction } },
                },
            },
            MuiDateTimePicker: {
                defaultProps: {
                    slotProps: {
                        openPickerButton: { size: "small" },
                        popper: { dir: theme.direction },
                    },
                },
            },
            MuiDayCalendar: {
                styleOverrides: {
                    header: {
                        "& .MuiTypography-caption": {
                            fontStyle: "normal",
                        },
                    },
                    weekContainer: {
                        button: {
                            fontStyle: "normal",
                        },
                    },
                },
            },
            MuiDialogActions: {
                styleOverrides: {
                    root: {
                        margin: spacing(1),
                    },
                },
            },
            MuiFormControl: {
                styleOverrides: {
                    root: {
                        marginBottom: spacing(1),
                    },
                },
            },
            MuiFormHelperText: {
                defaultProps: {
                    variant: "standard",
                },
                styleOverrides: {
                    root: {
                        ...subtitle2,
                        marginTop: 0,
                        marginBottom: spacing(0.5),
                        "&.Mui-error": {
                            borderRadius: shape.borderRadius,
                            color: palette.custom.errorText,
                            fontStyle: "italic",
                            backgroundColor: palette.background.default,
                        },
                    },
                },
            },
            MuiFormLabel: {
                styleOverrides: {
                    root: {
                        // Override disabled background color that gets applied
                        backgroundColor: "transparent",
                        margin: spacing(0.5, 0),
                        color: "currentColor",
                        // Clear InlineHelp button
                        paddingRight: pxToRem(32),
                        lineHeight: 1.4,
                        fontWeight: fontWeightMedium,
                        "&.Mui-error, &.Mui-error.Mui-focused": {
                            borderRadius: shape.borderRadius,
                            color: palette.text.primary,
                            backgroundColor: palette.background.default,
                            // Error icon background circle
                            "& .GcxFormLabel-errorIconBackground": {
                                display: "inline-flex",
                                width: pxToRem(22),
                                height: pxToRem(22),
                                justifyContent: "center",
                                marginRight: "4px",
                                alignItems: "center",
                                borderRadius: "50%",
                                backgroundColor: palette.custom.errorText,
                            },
                        },
                        "&.Mui-error .GcxFormLabel-errorIcon": {
                            margin: "0px 0px 2px 0px",
                            color: palette.custom.errorBackground,
                        },
                        "&.Mui-focused": {
                            color: "currentColor",
                        },
                    },
                    asterisk: {
                        "&.Mui-error": {
                            color: "white",
                        },
                    },
                },
            },
            MuiIconButton: {
                defaultProps: {
                    focusRipple: false,
                },
                styleOverrides: {
                    root: {
                        border: "1px solid transparent",
                        "&.Mui-focusVisible": {
                            borderRadius: "50%",
                            backgroundColor: "transparent",
                            ...getFocusHighlightStyles(theme),
                        },
                    },
                    sizeSmall: {
                        padding: spacing(0.375),
                    },
                },
            },
            MuiInput: {
                defaultProps: {
                    disableUnderline: true,
                },
                styleOverrides: {
                    formControl: {
                        "label + &": {
                            marginTop: "0",
                        },
                    },
                    input: {
                        borderRadius: shape.borderRadius,
                        // Fixes issues with our font where letters such as `g` would get cut off
                        lineHeight: "1.3125em",
                    },
                    root: {
                        border: `1px solid ${palette.grey[400]}`,
                        borderRadius: shape.borderRadius,
                        padding: 0,
                        "&.Mui-focused, &.Mui-error.Mui-focused": {
                            borderColor: blue.A100,
                            boxShadow: "none",
                        },
                        "&.Mui-error, &.Mui-error.Mui-focused": {
                            backgroundColor: "transparent",
                            borderWidth: "2px",
                            borderColor: palette.error.main,
                            boxSizing: "border-box",
                            boxShadow: `0 0 0 1px ${palette.custom.darkModeEmphasis} inset, 0 0 0 1px ${palette.custom.darkModeEmphasis}`,
                        },
                        "&.Mui-disabled": {
                            borderColor: palette.grey[300],
                            backgroundColor: palette.custom.greyBackground,
                        },
                    },
                    inputMultiline: {
                        padding: `${spacing(0.625)} ${spacing(1)}`,
                    },
                },
            },
            MuiInputAdornment: {
                styleOverrides: {
                    positionEnd: {
                        marginRight: 8,
                    },
                    positionStart: {
                        marginLeft: 8,
                    },
                },
            },
            MuiInputBase: {
                styleOverrides: {
                    input: {
                        // Let the height expand to fit content
                        height: "auto",
                        padding: `${spacing(0.625)} ${spacing(1)}`,
                    },
                },
            },
            MuiLink: {
                defaultProps: {
                    underline: "hover",
                },
            },
            MuiListItemButton: {
                styleOverrides: {
                    root: getMenuItemStyles(theme),
                },
            },
            MuiListItem: {
                styleOverrides: {
                    root: {
                        variants: [
                            {
                                // HACK: The button prop is deprecated in MUI 5 in favor
                                // of using ListItemButton. This applies the same
                                // styling for any existing code still using this
                                // property.
                                props: { button: true },
                                style: {
                                    ...getMenuItemStyles(theme),
                                },
                            },
                        ],
                        "&.MuiListItem-secondaryAction, &>.MuiListItemButton-root": {
                            // Disable the right padding of 48px that MUI uses
                            // to make room for secondary actions, since we use
                            // flex for those rather than absolute positioning
                            // (see MuiListItemSecondaryAction overrides). Note
                            // that there are two distinct cases we need to
                            // handle, depending on whether a
                            // <ListItemSecondaryAction> element is used
                            // explicitly, or whether the ListItem's
                            // `secondaryAction` prop is used.
                            paddingRight: spacing(1),
                        },
                    },
                    container: {
                        // A wrapper element with this class is added around a
                        // list item when a <ListItemSecondaryAction> child is
                        // present. Use flex to position the secondary actions
                        // next to the list item content rather than absolute
                        // positioning so that nothing overlaps.
                        display: "flex",
                        alignItems: "center",
                    },
                },
            },
            MuiListItemSecondaryAction: {
                styleOverrides: {
                    root: {
                        // We use flex instead of absolute positioning so that
                        // secondary buttons don't overlap the item text.
                        position: "static",
                        transform: "unset",
                        flex: "0 0 auto",
                        paddingRight: spacing(1),
                    },
                },
            },
            MuiListItemText: {
                styleOverrides: {
                    primary: {
                        fontWeight: 500,
                    },
                },
            },
            MuiListSubheader: {
                styleOverrides: {
                    sticky: {
                        backgroundColor: "inherit",
                    },
                },
            },
            MuiMenuItem: {
                styleOverrides: {
                    root: {
                        [theme.breakpoints.up("sm")]: {
                            // Reset to make height consistent with ListItems
                            minHeight: 48,
                        },
                        paddingTop: spacing(1),
                        paddingBottom: spacing(1),
                        ...getMenuItemStyles(theme),
                    },
                    dense: {
                        [theme.breakpoints.up("sm")]: {
                            minHeight: 36,
                        },
                        paddingTop: spacing(0.5),
                        paddingBottom: spacing(0.5),
                        "& .MuiListItemIcon-root svg": {
                            // MUI sets this to 1.25rem, which ignores the
                            // theme's `htmlFontSize` setting. Override with
                            // pxToRem to get the correct height regardless of
                            // HTML font size.
                            fontSize: pxToRem(20),
                        },
                    },
                },
            },
            MuiModal: {
                defaultProps: {
                    dir: theme.direction,
                },
            },
            MuiNativeSelect: {
                defaultProps: {
                    variant: "standard",
                },
                styleOverrides: {
                    select: {
                        ...getSelectStyles(theme),
                    },
                },
            },
            MuiPagination: {
                defaultProps: {
                    color: "primary",
                },
            },
            MuiRadio: {
                defaultProps: {
                    color: "secondary",
                    focusRipple: false,
                },
            },
            MuiSelect: {
                defaultProps: {
                    variant: "standard",
                },
                styleOverrides: {
                    select: {
                        ...getSelectStyles(theme),
                    },
                },
            },
            MuiSlider: {
                styleOverrides: {
                    valueLabel: {
                        // Set the minimum height to avoid it jumping around
                        // when the label does not have a value.
                        minHeight: pxToRem(28),
                    },
                },
            },
            MuiSwitch: {
                defaultProps: {
                    color: "secondary",
                    focusRipple: false,
                },
                styleOverrides: {
                    root: {
                        width: 52,
                        height: 42,
                        padding: 10,
                        // 'on' state indicator
                        "&::before": {
                            content: "'•'",
                            position: "absolute",
                            left: 14,
                            top: 7,
                            fontSize: 18,
                        },
                    },
                    colorPrimary: {
                        "&.Mui-checked": {
                            color: palette.custom.negativeSpace,
                        },
                    },
                    colorSecondary: {
                        "&.Mui-checked": {
                            color: palette.custom.negativeSpace,
                        },
                    },
                    sizeSmall: {
                        width: 36,
                        height: 30,
                        padding: 6,
                        marginLeft: 4,
                        // 'on' state indicator
                        "&::before": {
                            left: 8,
                            top: 4,
                            fontSize: 14,
                        },
                        "& .MuiSwitch-thumb": {
                            width: 12,
                            height: 12,
                        },
                        "& .MuiSwitch-switchBase": {
                            padding: 5,
                            "&.Mui-checked": {
                                transform: "translateX(33%)",
                            },
                        },
                    },
                    switchBase: {
                        top: 3,
                        left: 2,
                        border: "1px solid transparent",
                        "&.Mui-checked": {
                            transform: "translateX(33%)",
                        },
                        "&.Mui-checked + .MuiSwitch-track": {
                            opacity: 1,
                        },
                    },
                    thumb: {
                        width: 16,
                        height: 16,
                        borderRadius: 2,
                        backgroundColor: palette.custom.negativeSpace,
                    },
                    track: {
                        borderRadius: shape.borderRadius,
                    },
                },
            },
            MuiTab: {
                defaultProps: {
                    focusRipple: false,
                },
                styleOverrides: {
                    root: {
                        backgroundColor: palette.custom.greyBackground,
                        borderStyle: "solid",
                        borderWidth: 1,
                        borderColor: palette.grey[400],
                        borderLeftWidth: 0,
                        height: "unset",
                        minHeight: isDense ? 30 : 40,
                        padding: `${spacing(0.75)} ${spacing(1.5)}`,
                        textTransform: "none",
                        [theme.breakpoints.up("sm")]: {
                            minWidth: 72,
                        },
                    },
                    labelIcon: {
                        minHeight: 72,
                    },
                    textColorPrimary: {
                        "&.Mui-selected": {
                            backgroundColor: "unset",
                        },
                    },
                },
            },
            MuiTableCell: {
                styleOverrides: {
                    root: {
                        border: `1px solid ${palette.grey[400]}`,
                        padding: spacing(1.5),
                        overflowWrap: "anywhere",
                        "&:last-child": {
                            paddingRight: undefined,
                        },
                    },
                    head: {
                        // Chosen to match PanelGroup in react-ui-internal.
                        backgroundColor: palette.custom.greyBackground,
                        fontWeight: "bold",
                        fontSize: pxToRem(14),
                    },
                },
            },
            MuiTableRow: {
                styleOverrides: {},
            },
            MuiTabs: {
                styleOverrides: {
                    root: {
                        height: "auto",
                        minHeight: 36,
                    },
                    indicator: {
                        top: "0",
                        bottom: "auto",
                        height: 4,
                    },
                    flexContainer: {
                        borderColor: palette.grey[400],
                        borderStyle: "solid",
                        borderWidth: 0,
                        borderBottomWidth: 1,
                        height: "unset",
                        minHeight: isDense ? 30 : 40,
                        "& >div >button": {
                            marginBottom: -1,
                            borderLeftWidth: 1,
                            borderRightWidth: 0,
                            "&[aria-selected=true]": {
                                borderBottomColor: palette.background.default,
                            },
                        },
                        "& >div:last-of-type >button": {
                            borderRightWidth: 1,
                        },
                    },
                    flexContainerVertical: {
                        height: "100%",
                        width: "fit-content",
                        borderBottomWidth: 0,
                        borderRightWidth: 1,
                        "& >div >button": {
                            marginBottom: 0,
                            marginRight: -1,
                            borderBottomWidth: 0,
                            borderRightWidth: 1,
                            maxWidth: "10em",
                            width: "calc(100% + 1px)",
                            "&[aria-selected=true]": {
                                borderRightColor: palette.background.default,
                                borderBottomColor: palette.grey[400],
                            },
                        },
                        "& >div:last-of-type >button": {
                            borderBottomWidth: 1,
                        },
                    },
                    scrollButtons: {
                        border: `1px solid ${palette.grey[400]}`,
                    },
                    vertical: {
                        minWidth: "fit-content",
                        "& .MuiTabs-indicator": {
                            width: 4,
                            left: 0,
                        },
                    },
                },
            },
            MuiTextField: {
                defaultProps: { variant: "standard" },
            },
            MuiTimePicker: {
                defaultProps: {
                    slotProps: {
                        openPickerButton: { size: "small" },
                        popper: { dir: theme.direction },
                    },
                },
            },
            MuiToggleButton: {
                defaultProps: {
                    disableFocusRipple: true,
                },
                styleOverrides: {
                    root: {
                        // Same as MuiButton
                        padding: `${spacing(0.75)} ${spacing(2)}`,
                        height: undefined,
                        color: alpha(palette.text.primary, 0.7),
                        // Border color from Button contained
                        borderColor: palette.grey[400],
                        textTransform: "none",
                        transition: transitions.create(["background-color"], {
                            duration: transitions.duration.short,
                        }),
                        "&:hover": {
                            backgroundColor: alpha(palette.text.primary, palette.action.hoverOpacity),
                        },
                        "&.Mui-focusVisible": {
                            ...getFocusHighlightStyles(theme),
                            "&:active": {
                                // Taken from Button root's focusVisible in createTheme
                                backgroundColor: alpha(palette.grey.A200, 0.5),
                            },
                            "&.Mui-selected:active": {
                                // Taken from Button raised's focusVisible in createTheme
                                backgroundColor: palette.grey[500],
                            },
                        },
                        "&.Mui-selected": {
                            // Color and BackgroundColor from default Button contained style
                            color: palette.getContrastText(palette.grey[300]),
                            backgroundColor: palette.grey[300],
                            "&:hover": {
                                backgroundColor: palette.grey.A100,
                            },
                            "& + &": {
                                // Prevent shifting when keyboard focusing or clicking button with mouse
                                borderLeft: undefined,
                            },
                        },
                        "&.MuiToggleButton-primary": {
                            // Copy primary outlined button style.
                            color: palette.primary.main,
                            borderColor: alpha(palette.primary.main, 0.5),
                            "&:hover": {
                                // The !important hack overrides other rules that normally hide the left/right border.
                                borderColor: `${palette.primary.main} !important`,
                                backgroundColor: alpha(palette.primary.main, palette.action.hoverOpacity),
                            },
                            "&.Mui-selected": {
                                // Copy primary contained button style.
                                color: palette.primary.contrastText,
                                backgroundColor: palette.primary.main,
                                "&:hover": {
                                    backgroundColor: palette.primary.dark,
                                },
                            },
                        },
                        "&.MuiToggleButton-secondary": {
                            // Copy secondary outlined button style.
                            color: palette.secondary.main,
                            borderColor: alpha(palette.secondary.main, 0.5),
                            "&:hover": {
                                // The !important hack overrides other rules that normally hide the left/right border.
                                borderColor: `${palette.secondary.main} !important`,
                                backgroundColor: alpha(palette.secondary.main, palette.action.hoverOpacity),
                            },
                            "&.Mui-selected": {
                                // Copy secondary contained button style.
                                color: palette.secondary.contrastText,
                                backgroundColor: palette.secondary.main,
                                "&:hover": {
                                    backgroundColor: palette.secondary.dark,
                                },
                            },
                        },
                    },
                },
            },
            MuiToggleButtonGroup: {
                styleOverrides: {
                    grouped: {
                        "&:not(:first-child)": {
                            // Prevent shifting when keyboard focusing or clicking button with mouse
                            marginLeft: undefined,
                        },
                    },
                },
            },
        },
    };
}
function createTheme(options = {}) {
    const { dense = false, ...overrides } = options;
    // If the incoming options override either the fontSize or htmlFontSize,
    // these need to be specified when creating the theme so that we end up
    // with the correct implementation of pxToRem().
    const { fontSize = 14, htmlFontSize = 16 } = options.typography ?? {};
    // Populate a "default" MUI theme with our base colors and typography settings.
    // This will be used to create a theme options object that can take into
    // account the default values MUI creates.
    const defaultTheme = createMuiTheme({
        palette: { mode: options?.palette?.mode ?? "light", ...defaultPalette },
        typography: { fontSize, htmlFontSize },
    });
    // Merge the VertiGIS Studio theme defaults with any user specified theme options
    const mergedOptions = deepAssign({}, getDefaultOptions(defaultTheme), dense ? defaultDenseOptions : {}, overrides);
    const createdTheme = createMuiTheme(mergedOptions);
    // The component overrides use the final palette and so must be applied
    // last.
    return overrideTheme(createdTheme, getComponentOverrides(createdTheme, dense));
}
export default createTheme;
/**
 * Merges theme overrides with a base theme.
 *
 * @param theme The theme to override.
 * @param overrides New theme settings that will be deeply merged with the
 *   original theme.
 * @returns A new theme that is the merger of the original theme plus overrides.
 */
export function overrideTheme(theme, overrides) {
    return deepAssign({}, theme, overrides);
}
/**
 * A recursive version of Object.assign().
 */
function deepAssign(target, ...sources) {
    if (target === null || target === undefined) {
        throw new TypeError("Cannot convert undefined or null to object.");
    }
    const result = Object(target);
    sources.forEach(source => {
        if (!source) {
            return;
        }
        const sourceObj = Object(source);
        for (const key of Object.keys(sourceObj)) {
            const sourceValue = sourceObj[key];
            if ((result[key] && isPlainObject(result[key])) || Array.isArray(result[key])) {
                deepAssign(result[key], sourceValue);
            }
            else if (Array.isArray(sourceValue)) {
                result[key] = deepAssign([], sourceValue);
            }
            else if (isPlainObject(sourceValue)) {
                result[key] = deepAssign({}, sourceValue);
            }
            else {
                result[key] = sourceValue;
            }
        }
    });
    return result;
}
/**
 * Determines whether the argument is a plain instance of Object.
 *
 * @param value The value to check.
 */
function isPlainObject(value) {
    return (typeof value === "object" &&
        !Array.isArray(value) &&
        !!value &&
        Object.getPrototypeOf(value) === Object.prototype);
}
