import { useCallback, useEffect, useRef } from "react";
/**
 * Debounces a function, i.e. creates a wrapper that only invokes the original
 * once a certain amount of time has elapsed without being invoked.
 *
 * @param func The original function.
 * @param wait The amount of time to wait for inactivity (in milliseconds).
 * @param immediate If true, trigger the function on the leading edge, instead
 *   of the trailing.
 * @param abortSignal Optionally specify an abortSignal for the throttled
 *   function.
 */
export function debounce(func, wait, immediate, abortSignal) {
    let timeout;
    const cleanup = () => {
        window.clearTimeout(timeout);
        abortSignal?.removeEventListener("abort", cleanup);
        timeout = undefined;
    };
    return (...args) => {
        const later = () => {
            cleanup();
            // Do not execute here if 'immediate=true` this function is just a
            // timer to block execution until completed.
            if (!immediate && !abortSignal?.aborted) {
                func(...args);
            }
        };
        if (abortSignal?.aborted) {
            return;
        }
        const callNow = immediate && !timeout;
        cleanup();
        if (callNow) {
            func(...args);
        }
        abortSignal?.addEventListener("abort", cleanup);
        timeout = window.setTimeout(later, wait);
    };
}
/**
 * Same as `debounce()`, but optimized for animation and rendering.
 *
 * @param func The function to debounce.
 * @param abortSignal Optionally specify an abortSignal for the throttled
 *   function.
 */
export function debounceAnimation(func, abortSignal) {
    let animationFrame;
    const cleanup = () => {
        cancelAnimationFrame(animationFrame);
        abortSignal?.removeEventListener("abort", cleanup);
    };
    return (...args) => {
        cleanup();
        if (abortSignal?.aborted) {
            return;
        }
        animationFrame = requestAnimationFrame(() => {
            abortSignal?.removeEventListener("abort", cleanup);
            if (!abortSignal?.aborted) {
                func(...args);
            }
        });
        abortSignal?.addEventListener("abort", cleanup);
    };
}
/**
 * Returns a debounced and memoized version of the callback that only changes if
 * one of the `deps` has changed.
 *
 * @param callback The function to debounce and memoize.
 * @param timeout The debounce delay.
 * @param deps The dependencies of the callback.
 */
export function useDebounce(callback, timeout, deps) {
    // Abort any debounced calls on unmount.
    const abortController = useRef(new AbortController());
    useEffect(() => () => {
        abortController.current.abort();
        // `React.StrictMode` causes all components to mount twice in
        // development. This will break debounced functions unless
        // we refresh the abort controller here.
        abortController.current = new AbortController();
    }, []);
    // eslint-disable-next-line react-hooks/exhaustive-deps
    return useCallback(debounce(callback, timeout, false, abortController.current.signal), deps);
}
