import { Sanitizer } from "@esri/arcgis-html-sanitizer";
import { escapeAttrValue, safeAttrValue, friendlyAttrValue } from "xss";
/**
 * Sanitize a string of unsafe HTML to eliminate XSS (cross-site scripting)
 * security risks. This method allows tags and standard attributes, however
 * `data-` attributes are not allowed. Script tags are not allowed either.
 *
 * @param unsafeHtml String of HTML to be scrubbed clean.
 * @returns Safe HTML that has been sanitized.
 */
export function sanitizeHtml(unsafeHtml) {
    const checkUris = (tag, attr, value, cssFilter) => {
        if (attr === "href" || attr === "src") {
            // Override the default implementation to allow relative URLs.
            if (!value) {
                return "";
            }
            try {
                // Unescape attribute value first.
                let href = friendlyAttrValue(value);
                // Remove any surrounding grave accents and embedded whitespace
                // that can fool the parsing in the URL class, potentially
                // allowing unsafe URLs through.
                href = href.replace(/^`|`$|\s+/g, "");
                // The base URL can be anything -- it's just there to set the
                // protocol to "https" for all relative URLs.
                const url = new URL(href, "https://vertigis.com/");
                if (url.protocol === "http:" ||
                    url.protocol === "https:" ||
                    url.protocol === "mailto:") {
                    return escapeAttrValue(value);
                }
            }
            catch {
                // Ignore and let it be stripped out below. It's probably
                // safest to err on the side of disallowing URLs that cannot be
                // parsed.
            }
            return "";
        }
        return safeAttrValue(tag, attr, value, cssFilter);
    };
    const options = {
        safeAttrValue: checkUris,
        // We are mainly relying on the default HTML & CSS whitelists provided by Esri's
        // sanitization library (some element/attributes are maintained for backwards compatibility)
        // and only overriding some options (since we are passing `true` as the second parameter).
        whiteList: {
            a: ["title"],
            img: ["title"],
            ol: ["start"],
            thead: [],
        },
        stripIgnoreTag: true,
        stripIgnoreTagBody: true,
        allowCommentTag: false,
        css: {
            whiteList: {
                "background-image": false,
            },
        },
    };
    return new Sanitizer(options, true).sanitize(unsafeHtml);
}
/**
 * Strip all HTML out of a string.
 *
 * @param html String of text which will have its HTML stripped away.
 * @returns A plain text string without any HTML markup.
 */
export function stripHtml(html) {
    if (!html) {
        return html;
    }
    // Sanitize any HTML that is present, to make it safe (from XSS risks).
    const safe = sanitizeHtml(html);
    // Create a temporary div (using the browser to strip away HTML content).
    const div = document.createElement("div");
    div.innerHTML = safe;
    return div.innerText;
}
