/**
 * Options for the markdownToHtml() function.
 */
export interface MarkdownToHtmlOptions {
    /**
     * If specified, produces HTML that is valid to place within an HTML element
     * that only supports inline content (called "phrasing content" in the HTML5
     * spec). Example elements that only support phrasing content are <span>
     * and<h2>. In this mode, any block-level elements resulting from the
     * Markdown conversion will be stripped. The default is `false`.
     */
    inline?: boolean;
    /**
     * If true, any HTML in the markdown will be escaped rather than passed
     * through directly to the output (since Markdown is a superset of HTML).
     * This is useful in situations where you wish to only support pure Markdown
     * syntax without also supporting HTML syntax. The default is `false`.
     *
     * IMPORTANT: The resulting HTML still needs to be sanitized whether or not
     * this option is set.
     */
    escapeHtml?: boolean;
    /**
     * An optional value to apply to external hyperlinks via the `target`
     * attribute.
     *
     * If this is omitted, links may still open in a separate target if the host
     * page specifies a `target` via the `<base>` element in its `<head>`.
     */
    linkTarget?: string;
}
/**
 * Converts markdown text into HTML. The resulting HTML is sanitized and safe to
 * insert into the DOM.
 *
 * @param markdown The markdown text to convert.
 * @param options Options that affect how the text is converted.
 */
export declare function markdownToSafeHtml(markdown: string, options?: MarkdownToHtmlOptions): string;
/**
 * Converts markdown text into HTML.
 *
 * NOTE: The resulting HTML is NOT sanitized by this function. It is the
 * caller's responsibility to sanitize the resulting HTML prior to inserting it
 * into the DOM. You must do this even if `escapeHtml` is set to true.
 *
 * @param markdown The markdown text to convert.
 * @param options Options that affect how the text is converted.
 */
export declare function markdownToHtml(markdown: string | undefined, options?: MarkdownToHtmlOptions): string;
