import escapeHtml from "lodash.escape";
import { marked } from "marked";
import { sanitizeHtml } from "./html.js";
/**
 * A custom marked renderer that supports a few additional features of our own:
 *
 * - If `escapeHtml` is true, it escapes HTML in the original markdown.
 * - If `linkTarget` is set, it adds a `target` attribute to all external links.
 */
class CustomRenderer extends marked.Renderer {
    escapeHtml = false;
    linkTarget = undefined;
    constructor(escapeHtml, linkTarget) {
        super();
        this.escapeHtml = escapeHtml;
        this.linkTarget = linkTarget;
    }
    html(html) {
        return this.escapeHtml ? escapeHtml(html) : html;
    }
    link(href, title, text) {
        if (this.linkTarget) {
            const localLink = href?.startsWith(`${location.protocol}//${location.hostname}`);
            const html = super.link(href, title, text);
            return localLink ? html : html.replace(/^<a /, `<a target="${this.linkTarget}" `);
        }
        else {
            return super.link(href, title, text);
        }
    }
}
/**
 * Converts markdown text into HTML. The resulting HTML is sanitized and safe to
 * insert into the DOM.
 *
 * @param markdown The markdown text to convert.
 * @param options Options that affect how the text is converted.
 */
export function markdownToSafeHtml(markdown, options) {
    return sanitizeHtml(markdownToHtml(markdown, options));
}
/**
 * Converts markdown text into HTML.
 *
 * NOTE: The resulting HTML is NOT sanitized by this function. It is the
 * caller's responsibility to sanitize the resulting HTML prior to inserting it
 * into the DOM. You must do this even if `escapeHtml` is set to true.
 *
 * @param markdown The markdown text to convert.
 * @param options Options that affect how the text is converted.
 */
export function markdownToHtml(markdown, options) {
    if (!markdown) {
        return "";
    }
    const { inline, escapeHtml, linkTarget } = {
        inline: false,
        escapeHtml: false,
        linkTarget: undefined,
        ...options,
    };
    const renderer = new CustomRenderer(escapeHtml, linkTarget);
    const html = marked(markdown, { renderer });
    return inline ? stripBlockElements(html) : html;
}
/**
 * Strips all HTML elements that are not suitable for placing within an inline
 * HTML element.
 *
 * @param html The HTML to process.
 */
function stripBlockElements(html) {
    // Derived from here: https://html.spec.whatwg.org/multipage/dom.html#phrasing-content
    return html.replace(/(?:<(\/?))(?!(?:\/|a|abbr|area|audio|b|bdi|bdo|br|button|canvas|cite|code|data|datalist|del|dfn|em|embed|i|iframe|img|input|ins|kbd|label|link|map|mark|math|meta|meter|noscript|object|output|picture|progress|q|ruby|s|samp|script|select|slot|small|span|strong|sub|sup|svg|template|textarea|time|u|var|video|wbr)\b)[^>]*>/gi, (match, s1) => (s1 === "/" ? " " : ""));
}
