import { useCallback, useLayoutEffect, useEffect, useRef, useState } from "react";
/**
 * Generate a random 10 digit id.
 *
 * @param prefix An optional prefix to prepend to the generated id.
 * @internal
 */
export function _generateId(prefix) {
    const randomId = Math.random().toString(36).substr(2, 10);
    if (!prefix) {
        return randomId;
    }
    return `${prefix}-${randomId}`;
}
/**
 * Produces a unique ID that can be used for the HTML `id` attribute. Will use
 * the provided value if defined (typically obtained from the calling
 * component's `id` prop), otherwise one will be generated. The ID will remain
 * the same between renders (even if `idProp` changes).
 */
export function useId(idProp) {
    const [id] = useState(() => idProp ?? _generateId());
    return id;
}
/**
 * Combines multiple refs into a single ref.
 *
 * Useful when a component needs a ref to one of its own elements, but also
 * supports forwarding a ref from its parent component.
 */
export function mergeRefs(...refs) {
    return (value) => {
        refs.forEach(ref => {
            if (typeof ref === "function") {
                ref(value);
            }
            else if (ref) {
                ref.current = value;
            }
        });
    };
}
/**
 * Returns the previous value of updated props or state.
 *
 * @param value The original value of the property.
 */
export function usePrevious(value) {
    const ref = useRef();
    useEffect(() => {
        ref.current = value;
    }, [value]);
    return ref.current;
}
/**
 * A simple, unoptimized implementation of useResizeObserver.
 *
 * @param target The node to observe.
 * @param callback Called when a change is observed.
 */
export function useResizeObserver(target, callback) {
    const observer = useRef();
    const disconnect = useCallback(() => observer.current?.disconnect(), []);
    const observe = useCallback(() => {
        if (window.ResizeObserver) {
            observer.current = new ResizeObserver(([entry]) => callback(entry));
            if (target) {
                observer.current.observe(target);
            }
        }
    }, [callback, target]);
    useLayoutEffect(() => {
        observe();
        return () => disconnect();
    }, [disconnect, observe]);
}
/**
 * Gets the dimensions of an element.
 *
 * @param target A ref to the target element.
 */
export function useContentRect(target) {
    const [rect, setRect] = useState();
    const [element, setElement] = useState();
    useLayoutEffect(() => {
        const el = Object.prototype.hasOwnProperty.call(target, "current")
            ? target.current
            : target;
        setRect(el?.getBoundingClientRect());
        setElement(el);
    }, [target]);
    useResizeObserver(element, entry => {
        if (entry.contentRect.height !== rect?.height || entry.contentRect.width !== rect?.width) {
            setRect(entry.contentRect);
        }
    });
    return rect ?? {};
}
/**
 * Executes a function once when the component is first mounted. The function
 * will never run again unless the component is unmounted and subsequently
 * mounted again.
 *
 * @param f The function to execute. If the function is async, it's the caller's
 *   responsibility to verify that the component is still mounted after awaiting
 *   any promises (e.g. using `useIsMounted`).
 */
export function useMount(f) {
    useEffect(() => {
        // This wrapper prevents the promise from being returned, if there is
        // one.
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        f();
        // It's intended that this hook runs only once on mount.
        //
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
}
/**
 * Executes a function once when the component is dismounted.
 *
 * @param f The function to execute.
 */
export function useDismount(f) {
    useEffect(() => f, 
    // It's intended that this hook runs only once on dismount.
    //
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
}
