/**
 * Throttles a function so it can only be fired once per given period of
 * milliseconds. If the function is called during the throttling period, the
 * last instance of the function being called will be run once the throttling
 * period ends. The callback is always invoked asynchronously.
 *
 * @param func The function to be throttled.
 * @param msBetweenRequests The minimum time between requests.
 */
export function throttle(func, msBetweenRequests) {
    let isScheduled = false;
    return (...args) => {
        if (!isScheduled) {
            window.setTimeout(() => {
                isScheduled = false;
                // eslint-disable-next-line @typescript-eslint/no-floating-promises
                func(...args);
            }, msBetweenRequests);
            isScheduled = true;
        }
    };
}
/**
 * Same as `throttle()`, but optimized for animation and rendering.
 *
 * @param func The function to throttle.
 */
export function throttleAnimation(func) {
    let isScheduled = false;
    return (...args) => {
        if (!isScheduled) {
            requestAnimationFrame(() => {
                isScheduled = false;
                // eslint-disable-next-line @typescript-eslint/no-floating-promises
                func(...args);
            });
            isScheduled = true;
        }
    };
}
